use crate::{TallyState, Channel};
use std::sync::mpsc;
use std::sync::mpsc::{Sender, Receiver};
use std::thread;
use std::time::Duration;
use rand::prelude::*;

/// An Adaptor interfaces to a system providing tally information, e.g. a BlackMagic ATEM
pub trait Adaptor {
    /// Run the adaptor, returning a channel receiver which receives TallyStates
    fn run(&mut self) -> Receiver<TallyState>;
}

pub struct FakeAdaptor {
    numchannels: u8,
    thread: Option<thread::JoinHandle<()>>
}
impl FakeAdaptor {
    pub fn new(numchannels: u8) -> FakeAdaptor {
        return FakeAdaptor{
            numchannels: numchannels,
            thread: None
        }
    }

    fn _run(numchannels: u8, sender: Sender<TallyState>) {
        println!("Hello, I have {} channels", numchannels);
        loop {
            let state = TallyState{
                on_air: Channel(Some(rand::thread_rng().gen_range(0, numchannels))),
                preview: Channel(Some(rand::thread_rng().gen_range(0, numchannels))),
            };
            sender.send(state).unwrap();
            thread::sleep(Duration::new(rand::thread_rng().gen_range(0,5), 0));
        }
    }
}

impl Adaptor for FakeAdaptor {
    fn run(&mut self) -> Receiver<TallyState> {
        println!("arse");
        let (tx, rx): (Sender<TallyState>, Receiver<TallyState>) = mpsc::channel();
        let thread_numchannels = self.numchannels.clone();
        self.thread = Some(thread::spawn(move || {
            FakeAdaptor::_run(thread_numchannels, tx);
        }));
        return rx;
    }
}