use crate::TallyState;

use std::sync::mpsc;
use std::sync::mpsc::{Sender, Receiver};
use std::thread;

/// A Driver displays the Tallies on a physical tally system.
pub trait Driver {
    /// Run the driver, returning a channel sender which takes TallyStates
    fn run(&mut self) -> Sender<TallyState>;
}

pub struct FakeDriver {
    thread: Option<thread::JoinHandle<()>>
}

impl Driver for FakeDriver {
    fn run(&mut self) -> Sender<TallyState> {
        let (tx, rx): (Sender<TallyState>, Receiver<TallyState>) = mpsc::channel();
        self.thread = Some(thread::spawn(move || {
            FakeDriver::_run(rx);
        }));
        return tx;
    }
}

impl FakeDriver {
    pub fn new() -> FakeDriver {
        FakeDriver{
            thread: None
        }
    }

    fn _run(rx: Receiver<TallyState>) {
        loop {
            let state = rx.recv().unwrap();
            println!("{}", state);
        }
    }
}