mod adaptors;
mod drivers;
use crate::adaptors::{Adaptor, FakeAdaptor};
use crate::drivers::{Driver, FakeDriver};
use std::fmt;

#[derive(Clone)]
pub struct Channel(Option<u8>);

#[derive(Clone)]
pub struct TallyState {
    on_air: Channel,
    preview: Channel,
}

impl fmt::Display for Channel {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self.0 {
            Some(n) => write!(f, "{}", n),
            None => write!(f, "-"),
        }
    }
}

impl fmt::Display for TallyState {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "On Air: {}, Preview: {}", self.on_air, self.preview)
    }
}

struct Server<T: Adaptor, U: Driver> {
    adaptor: T,
    driver: U,
}

impl<T: Adaptor, U: Driver> Server<T, U> {
    fn run(&mut self) {
        let rx = self.adaptor.run();
        let tx = self.driver.run();
        loop {
            tx.send(rx.recv().unwrap()).unwrap();
        }
    }
}

fn main() {
    let mut s = Server {
        adaptor: FakeAdaptor::new(8),
        driver: FakeDriver::new(),
    };

    s.run();
}
